// ==============0ooo===================================================0ooo===========
// =  Copyright (C) 2014-2020 Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// ====================================================================================
// 
//  __      __      __
//  \ \    /  \    / /   [File name   ] vlb_top.v
//   \ \  / /\ \  / /    [Description ] Video line buffer based on FIFO
//    \ \/ /  \ \/ /     [Timestamp   ] Monday March 1 14:00:30 2021
//     \  /    \  /      [version     ] 1.0
//      \/      \/
//
// ==============0ooo===================================================0ooo===========
// Code Revision History :
// ----------------------------------------------------------------------------------
// Ver:    |  Author    | Mod. Date    | Changes Made:
// ----------------------------------------------------------------------------------
// V1.0    | Caojie     | 03/01/21     | Initial version 
// ----------------------------------------------------------------------------------
// ==============0ooo===================================================0ooo===========

`timescale 1ns / 1ps
    
module vlb_top
( 
    input                       I_rst_n           ,
    // video data input                              
    input                       I_vin0_clk        ,
    input                       I_vin0_de         ,
    input  [23:0]               I_vin0_data       ,
    // video data output                    
    input                       I_vout0_clk       ,
    input                       I_vout0_vs        ,//positive
    input                       I_vout0_de        ,
    output                      O_vout0_den       ,
    output [23:0]               O_vout0_data       
);

//==================================================================  
reg       fifo_wr_den;
reg[23:0] fifo_wr_data;

reg       fifo_rd_en;
reg       fifo_rd_en_d1;
wire[23:0]fifo_rd_data;

reg       fifo_rd_rst;
reg       vout_vs_d0;
reg       vout_vs_d1;
wire      vout_vs_rising;

//================================================================== 
// write fifo
always@(posedge I_vin0_clk or negedge I_rst_n)  
begin
    if (I_rst_n == 1'b0) 
        begin
            fifo_wr_den  <=  1'b0;
            fifo_wr_data <=  24'd0;
        end 
    else 
        begin
            fifo_wr_den  <=  I_vin0_de;
            fifo_wr_data <=  I_vin0_data;
        end
end

//----------------------
//read fifo
always@(posedge I_vout0_clk or negedge I_rst_n)  // write fifo
begin
    if (I_rst_n == 1'b0) 
        begin
            fifo_rd_en    <= 1'b0;
            fifo_rd_en_d1 <= 1'b0;
        end 
    else 
        begin
            fifo_rd_en    <= I_vout0_de;
            fifo_rd_en_d1 <= fifo_rd_en; 
        end
end

assign O_vout0_den  = fifo_rd_en_d1;
assign O_vout0_data = fifo_rd_data ;

//------------------------ 
fifo_read fifo_read_inst
(
    .Reset         (fifo_rd_rst   ),
    .WrClk         (I_vin0_clk    ),
    .RdClk         (I_vout0_clk   ),
    .Data          (fifo_wr_data  ),
    .WrEn          (fifo_wr_den   ),
    .RdEn          (fifo_rd_en    ),
    .Q             (fifo_rd_data  ),
    .Full          (              ),
    .Empty         (              )
);


//------------------------
//every frame, fifo reset
always@(posedge I_vout0_clk or negedge I_rst_n)
begin
    if(!I_rst_n)
        begin
            vout_vs_d0  <=  1'b1;
            vout_vs_d1  <=  1'b1;
        end
    else
        begin
            vout_vs_d0  <=  I_vout0_vs;
            vout_vs_d1  <=  vout_vs_d0;
        end
end

assign  vout_vs_rising = (~vout_vs_d1) && (vout_vs_d0);  //vs rising edge

always@(posedge I_vout0_clk or negedge I_rst_n)  // write fifo
begin
    if (I_rst_n == 1'b0) 
        begin
            fifo_rd_rst  <=  1'b1;
        end 
    else 
        begin
            fifo_rd_rst  <=  vout_vs_rising;
        end
end

endmodule   
  