// ==============0ooo===================================================0ooo===========
// =  Copyright (C) 2014-2019 Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// ====================================================================================
// 
//  __      __      __
//  \ \    /  \    / /   [File name   ] scaler_test_top.v
//   \ \  / /\ \  / /    [Description ] Scaler test module
//    \ \/ /  \ \/ /     [Timestamp   ] Friday Jul 17 14:00:30 2019
//     \  /    \  /      [version     ] 3.0
//      \/      \/
//
// ==============0ooo===================================================0ooo==========
// Code Revision History :
// ----------------------------------------------------------------------------------
// Ver:    |  Author    | Mod. Date    | Changes Made:
// ----------------------------------------------------------------------------------
// V1.0    | Caojie     | 07/17/19     | Initial version 
// ----------------------------------------------------------------------------------
// V1.1    | Caojie     | 10/12/19     | Scaler IP update 
// ----------------------------------------------------------------------------------
// V2.0    | Caojie     | 06/08/20     | Scaler IP update 
// ----------------------------------------------------------------------------------
// V3.0    | Caojie     | 11/11/21     | Scaler IP update 
// ----------------------------------------------------------------------------------
// ==============0ooo===================================================0ooo==========


module scaler_test_top
(
    input          I_clk           ,//50MHz
    input          I_rst_n         ,//I_KEY1
    output [3:0]   O_led           ,
    inout          IO_adv7513_scl,
    inout          IO_adv7513_sda,
    output         O_adv7513_clk ,
    output         O_adv7513_vs  ,
    output         O_adv7513_hs  ,
    output         O_adv7513_de  ,
    output [23:0]  O_adv7513_data,
    output [13:0]  O_ddr_addr   ,
    output [2:0]   O_ddr_ba     ,
    output         O_ddr_ras_n  ,
    output         O_ddr_cas_n  ,
    output         O_ddr_we_n   ,
    output         O_ddr_clk    ,
    output         O_ddr_clk_n  ,
    output         O_ddr_cke    ,
    output         O_ddr_odt    ,
    output         O_ddr_reset_n,
    output [1:0]   O_ddr_dqm    ,
    inout  [15:0]  IO_ddr_dq    ,
    inout  [1:0]   IO_ddr_dqs   ,
    inout  [1:0]   IO_ddr_dqs_n 
);

//=========================================================
//SRAM parameters
parameter ADDR_WIDTH          = 28;    //存储单元是byte，总容量=2^27*16bit = 2Gbit,增加1位rank地址，{rank[0],bank[2:0],row[13:0],cloumn[9:0]}
parameter DATA_WIDTH          = 128;   //与生成DDR3IP有关，此ddr3 2Gbit, x16， 时钟比例1:4 ，则固定128bit
parameter VIDEO_WIDTH         = 24;  

//===========================================
wire        rst_n1;
wire        rst_n2;

//--------------------------
reg  [31:0] run_cnt;
wire        running;

//----------------------------------------
//ddr
wire        dma_clk  ;  

//-------------------------------------------------
//memory interface
wire                      memory_clk         ;
wire                      pll_lock           ;
wire                      cmd_ready          ;
wire[2:0]                 cmd                ;
wire                      cmd_en             ;
wire[ADDR_WIDTH-1:0]      addr               ;
wire                      wr_data_rdy        ;
wire                      wr_data_en         ;//
wire                      wr_data_end        ;//
wire[DATA_WIDTH-1:0]      wr_data            ;   
wire[DATA_WIDTH/8-1:0]    wr_data_mask       ;   
wire                      rd_data_valid      ;  
wire                      rd_data_end        ;//unused 
wire[DATA_WIDTH-1:0]      rd_data            ;   
wire                      init_calib_complete; 

//===========================================
wire[12:0] vin_hres     ;
wire[12:0] vin_vres     ;
wire[12:0] vout_hres    ;
wire[12:0] vout_vres    ;
wire[23:0] hor_skfact   ;//shrink factor, unsigned fixed 8bit integer, 16bit fraction, (vin_hres/vout_hres)
wire[23:0] ver_skfact   ;//shrink factor, unsigned fixed 8bit integer, 16bit fraction, (vin_vres/vout_vres)

//-----------------------------------
wire        scaler_clk;
wire        pixel_clk; //65MHz
wire        pll_locked;

wire        ch0_vs_in  ;
wire        ch0_hs_in  ;
wire        ch0_de_in  ;
wire [ 7:0] ch0_data_r ;
wire [ 7:0] ch0_data_g ;
wire [ 7:0] ch0_data_b ;

//-------------------------------------------------
wire        scl_vin_vs;  
wire        scl_vin_de;
            
wire        sclout_vs  ;  
wire        sclout_de  ;
wire [23:0] sclout_data;

//--------------------------
wire        vval_de;
reg         vval_de_d1;     
reg         vval_de_d2;

reg  [31:0] delay_cnt;
reg         vlb_re;
wire        vlb_den;
wire [23:0] vlb_data; 
  
//-------------------------
//frame buffer in
wire                   ch0_vfb_clk_in ;
wire                   ch0_vfb_vs_in  ;
wire                   ch0_vfb_de_in  ;
wire [VIDEO_WIDTH-1:0] ch0_vfb_data_in;

//-------------------
//syn_code
wire                   syn_off0_re;  // ofifo read enable signal
wire                   syn_off0_vs;
wire                   syn_off0_hs;
                       
wire                   off0_syn_de  ;
wire [VIDEO_WIDTH-1:0] off0_syn_data;

//------------------------------------------
//ADV7513 I2c
wire         TX_EN;
wire [2:0]   WADDR;
wire [7:0]   WDATA;
wire         RX_EN;
wire [2:0]   RADDR;
wire [7:0]   RDATA;

//==============================================
//-------------------------------
assign O_led[0] = ~init_calib_complete;
assign O_led[1] = running;
assign O_led[2] = ~rst_n1;    
assign O_led[3] = ~I_rst_n;     


//==============================================
key_debounceN key_debounceN_inst0
(
    .clk       (I_clk), 
    .rst_n     (1'b1     ), 
    .key_in    (I_rst_n  ), 
    .key_n_out1(rst_n1   ),
    .key_n_out2(rst_n2   )
);

//===========================================================================
//generate pixel clock
pix_rpll pix_rpll_inst
(
    .clkout (pixel_clk   ), //output clkout 
    .lock   (            ), //output lock
    .reset  (!I_rst_n    ), //input reset
    .clkin  (I_clk       ) //input clkin
 );
 
//===================================================
//LED灯测试
always @(posedge I_clk or negedge I_rst_n)
begin
    if(!I_rst_n)
        run_cnt <= 32'd0;
    else if(run_cnt >= 32'd50_000_000)
        run_cnt <= 32'd0;
    else
        run_cnt <= run_cnt + 1'b1;
end

assign  running = (run_cnt < 32'd25_000_000) ? 1'b1 : 1'b0;
 
 //-----------------------------------------------------
//实时参数选择
assign vin_hres     = 13'd1920    ; //13'd1920    ;  //13'd1920    ;  //13'd1920    ;
assign vin_vres     = 13'd1080    ; //13'd1080    ;  //13'd1080    ;  //13'd1080    ;
assign vout_hres    = 13'd1600    ; //13'd1024    ;  //13'd1280    ;  //13'd1600    ;
assign vout_vres    = 13'd900     ; //13'd768     ;  //13'd720     ;  //13'd900     ;
assign hor_skfact   = 24'h013333  ; //24'h01e000  ;  //24'h018000  ;  //24'h013333  ;
assign ver_skfact   = 24'h013333  ; //24'h016800  ;  //24'h018000  ;  //24'h013333  ;

//===========================================================================
//testpattern
reg         vs_r;
reg  [9:0]  cnt_vs;

testpattern testpattern_inst
(
    .I_pxl_clk   (I_clk            ),//pixel clock
    .I_rst_n     (I_rst_n          ),//low active 
    .I_mode      ({1'b0,cnt_vs[8:7]}),//data select
    .I_sqr_width (16'd64           ),
    .I_single_r  (8'd0             ),
    .I_single_g  (8'd0             ),                  //74.25MHz   //148.5MHz 
    .I_single_b  (8'd255           ),                  //1280x720   //1920x1080
    .I_h_total   (16'd2200         ),//hor total time  // 16'd1650  // 16'd2200
    .I_h_sync    (16'd44           ),//hor sync time   // 16'd40    // 16'd44  
    .I_h_bporch  (16'd148          ),//hor back porch  // 16'd220   // 16'd148 
    .I_h_res     ({3'd0,vin_hres}  ),//hor resolution  // 16'd1280  // 16'd1920
    .I_v_total   (16'd1125         ),//ver total time  // 16'd750   // 16'd1125
    .I_v_sync    (16'd5            ),//ver sync time   // 16'd5     // 16'd5   
    .I_v_bporch  (16'd36           ),//ver back porch  // 16'd20    // 16'd36  
    .I_v_res     ({3'd0,vin_vres}  ),//ver resolution  // 16'd720   // 16'd1080
    .I_hs_pol    (1'b1             ),//HS polarity , 0:negative  1:positive
    .I_vs_pol    (1'b1             ),//VS polarity , 0:negative  1:positive
    .O_de        (ch0_de_in        ),   
    .O_hs        (ch0_hs_in        ),
    .O_vs        (ch0_vs_in        ),
    .O_data_r    (ch0_data_r       ),   
    .O_data_g    (ch0_data_g       ),
    .O_data_b    (ch0_data_b       )
);

always@(posedge I_clk or negedge I_rst_n)
begin
    if(!I_rst_n)
        vs_r<=1'b0;
    else
        vs_r<=ch0_vs_in;
end

always@(posedge I_clk or negedge I_rst_n)
begin
    if(!I_rst_n)
        cnt_vs<=10'd0;
    else if(vs_r && !ch0_vs_in) //vs falling edge
        cnt_vs<=cnt_vs+1'b1;
end

//=====================================================
//Scaler
assign scaler_clk = I_clk;

Scaler_Top Scaler_Top_inst
(
    .I_reset        (~I_rst_n              ),
    .I_sysclk       (scaler_clk            ),

    .I_param_update (1'b1                  ),
    .I_vin_hsize    (vin_hres              ),
    .I_vin_vsize    (vin_vres              ),
    .I_vout_hsize   (vout_hres             ),
    .I_vout_vsize   (vout_vres             ),
    .I_hor_skfactor (hor_skfact            ),
    .I_ver_skfactor (ver_skfact            ),
    
    .I_vin_clk      (I_clk                 ),
    .I_vin_vs_cpl   (ch0_vs_in             ),//positive                
    .I_vin_de_cpl   (ch0_de_in             ),                      
    .I_vin_data0_cpl(ch0_data_b            ), 
    .I_vin_data1_cpl(ch0_data_g            ), 
    .I_vin_data2_cpl(ch0_data_r            ), 
    .O_vout0_data   (sclout_data[ 7: 0]    ),                 
    .O_vout1_data   (sclout_data[15: 8]    ),   
    .O_vout2_data   (sclout_data[23:16]    ), 
                                           
    .O_vval_de      (vval_de               ),   
    .O_vval_data0   (                      ), 
    .O_vval_data1   (                      ), 
    .O_vval_data2   (                      ),     
                                
    .O_vout_vs      (sclout_vs             ),//positive                       
    .O_vout_de      (sclout_de             ) 
);

//======================================================
//因为Scaler缩小输出DE不连续，用line buffer fifo使其变成连续输出；
//如果video frame buffer支持非连续DE，可以不使用line buffer fifo。
//Scaler Live output DE and data are not continous, so make 
//them continous by line buffer fifo. If video frame buffer support
//noncontinous DE, can not use the line buffer fifo.
//-------------------------------------------------
vlb_top vlb_top_inst
( 
  .I_rst_n           (I_rst_n        ),
  .I_vin0_clk        (scaler_clk     ),
  .I_vin0_de         (sclout_de      ),
  .I_vin0_data       (sclout_data    ),
  .I_vout0_clk       (scaler_clk     ),
  .I_vout0_vs        (sclout_vs      ),//positive
  .I_vout0_de        (vlb_re         ),
  .O_vout0_den       (vlb_den        ),
  .O_vout0_data      (vlb_data       )
); 

always @(posedge scaler_clk or negedge I_rst_n)
begin
    if(!I_rst_n)
        begin
            vval_de_d1 <= 1'd0;
            vval_de_d2 <= 1'd0;
        end
    else
        begin
            vval_de_d1 <= vval_de;   
            vval_de_d2 <= vval_de_d1;   
        end 
end
  
always @(posedge scaler_clk or negedge I_rst_n)
begin
    if(!I_rst_n)
        delay_cnt <= 32'd0;
    else if(!vval_de_d2 & vval_de_d1) //rising edge
        delay_cnt <= 32'd0;
    else if(delay_cnt >= (44+148+vin_hres))//h_sync+h_bporch+h_res
        delay_cnt <= delay_cnt; 
    else
        delay_cnt <= delay_cnt+1'b1; 
end

always @(posedge scaler_clk or negedge I_rst_n)
begin
    if(!I_rst_n)
        vlb_re <= 1'd0;
    else if(delay_cnt >= (vin_hres-vout_hres+10) && delay_cnt <= (vin_hres+10-1)) //vin_hres-vout_hres=320
        vlb_re <= 1'b1; 
    else
        vlb_re <= 1'd0;
end

//==============================================  
    assign ch0_vfb_clk_in  = scaler_clk;
    assign ch0_vfb_vs_in   = ~sclout_vs;
    assign ch0_vfb_de_in   = vlb_den;
    assign ch0_vfb_data_in = vlb_data;//{r,g,b}  RGB888 
    
//=====================================================
Video_Frame_Buffer_Top Video_Frame_Buffer_Top_inst
( 
    .I_rst_n              (init_calib_complete ), //rst_n            ),
    .I_dma_clk            (dma_clk             ), //sram_clk         ),
    .I_wr_halt            (1'd0                ), //1:halt,  0:no halt
    .I_rd_halt            (1'd0                ), //1:halt,  0:no halt
    // video data input             
    .I_vin0_clk           (ch0_vfb_clk_in      ),
    .I_vin0_vs_n          (ch0_vfb_vs_in       ),//negative
    .I_vin0_de            (ch0_vfb_de_in       ),
    .I_vin0_data          (ch0_vfb_data_in     ),
    .O_vin0_fifo_full     (                    ),
    // video data output                       
    .I_vout0_clk          (pixel_clk           ),
    .I_vout0_vs_n         (~syn_off0_vs        ),//negative
    .I_vout0_de           (syn_off0_re         ),
    .O_vout0_den          (off0_syn_de         ),
    .O_vout0_data         (off0_syn_data       ),
    .O_vout0_fifo_empty   (                    ),
    // ddr write request                       
    .I_cmd_ready          (cmd_ready           ),
    .O_cmd                (cmd                 ),//0:write;  1:read
    .O_cmd_en             (cmd_en              ),
    .O_addr               (addr                ),//[ADDR_WIDTH-1:0]
    .I_wr_data_rdy        (wr_data_rdy         ),
    .O_wr_data_en         (wr_data_en          ),//
    .O_wr_data_end        (wr_data_end         ),//
    .O_wr_data            (wr_data             ),//[DATA_WIDTH-1:0]
    .O_wr_data_mask       (wr_data_mask        ),
    .I_rd_data_valid      (rd_data_valid       ),
    .I_rd_data_end        (rd_data_end         ),//unused 
    .I_rd_data            (rd_data             ),//[DATA_WIDTH-1:0]
    .I_init_calib_complete(init_calib_complete )
);  

//=====================================================
//ddr3 ip
ddr3_rpll ddr3_rpll_inst
(
    .clkout(memory_clk), //output clkout
    .lock  (pll_lock  ), //output lock
    .reset (~rst_n1   ), //input reset
    .clkin (I_clk     ) //input clkin
);

DDR3_Memory_Interface_Top DDR3_Memory_Interface_Top_inst 
(
    .clk                (I_clk              ),
    .memory_clk         (memory_clk         ),
    .pll_lock           (pll_lock           ),
    .rst_n              (rst_n1             ),   //rst_n
    .cmd_ready          (cmd_ready          ),
    .cmd                (cmd                ),
    .cmd_en             (cmd_en             ),
    .addr               (addr               ),
    .wr_data_rdy        (wr_data_rdy        ),
    .wr_data            (wr_data            ),
    .wr_data_en         (wr_data_en         ),
    .wr_data_end        (wr_data_end        ),
    .wr_data_mask       (wr_data_mask       ),
    .rd_data            (rd_data            ),
    .rd_data_valid      (rd_data_valid      ),
    .rd_data_end        (rd_data_end        ),
    .sr_req             (1'b0               ),
    .ref_req            (1'b0               ),
    .sr_ack             (                   ),
    .ref_ack            (                   ),
    .init_calib_complete(init_calib_complete),
    .clk_out            (dma_clk            ),
    .burst              (1'b1               ),
    // mem interface
    .ddr_rst            (                   ),
    .O_ddr_addr         (O_ddr_addr         ),
    .O_ddr_ba           (O_ddr_ba           ),
    .O_ddr_cs_n         (                   ),
    .O_ddr_ras_n        (O_ddr_ras_n        ),
    .O_ddr_cas_n        (O_ddr_cas_n        ),
    .O_ddr_we_n         (O_ddr_we_n         ),
    .O_ddr_clk          (O_ddr_clk          ),
    .O_ddr_clk_n        (O_ddr_clk_n        ),
    .O_ddr_cke          (O_ddr_cke          ),
    .O_ddr_odt          (O_ddr_odt          ),
    .O_ddr_reset_n      (O_ddr_reset_n      ),
    .O_ddr_dqm          (O_ddr_dqm          ),
    .IO_ddr_dq          (IO_ddr_dq          ),
    .IO_ddr_dqs         (IO_ddr_dqs         ),
    .IO_ddr_dqs_n       (IO_ddr_dqs_n       )
);

//=====================================================
//输出同步时序模块
wire out_de;
syn_gen syn_gen_inst      //实际工程  pixel_clk时钟信号 从管脚输入， 可省去PLL
(                                   
    .I_pxl_clk   (pixel_clk       ),//74.25MHz  //108MHz    //148.5MHz  
    .I_rst_n     (I_rst_n         ),//1280x720  //1600x900  //1920x1080 
    .I_h_total   (12'd1800        ),// 12'd1650 // 12'd1800 // 12'd2200 
    .I_h_sync    (12'd80          ),// 12'd40   // 12'd80   // 12'd44   
    .I_h_bporch  (12'd96          ),// 12'd220  // 12'd96   // 12'd148  
    .I_h_res     (12'd1600        ),// 12'd1280 // 12'd1600 // 12'd1920 
    .I_v_total   (12'd1000        ),// 12'd750  // 12'd1000 // 12'd1125 
    .I_v_sync    (12'd3           ),// 12'd5    // 12'd3    // 12'd5    
    .I_v_bporch  (12'd96          ),// 12'd20   // 12'd96   // 12'd36   
    .I_v_res     (12'd900         ),// 12'd720  // 12'd900  // 12'd1080 
    .I_rd_hres   (vout_hres[11:0] ),
    .I_rd_vres   (vout_vres[11:0] ),
    .I_hs_pol    (1'b1            ),//HS polarity , 0:negative  1:positive
    .I_vs_pol    (1'b1            ),//VS polarity , 0:negative  1:positive
    .O_rden      (syn_off0_re     ),
    .O_de        (out_de          ),   
    .O_hs        (syn_off0_hs     ),
    .O_vs        (syn_off0_vs     )
);

    localparam N = 8; //delay N clocks
                            
    reg  [N-1:0]  Pout_hs_dn   ;
    reg  [N-1:0]  Pout_vs_dn   ;
    reg  [N-1:0]  Pout_de_dn   ;
    
    always@(posedge pixel_clk or negedge I_rst_n)
    begin
        if(!I_rst_n)
            begin                          
                Pout_hs_dn  <= {N{1'b1}};
                Pout_vs_dn  <= {N{1'b1}}; 
                Pout_de_dn  <= {N{1'b0}}; 
            end
        else 
            begin                          
                Pout_hs_dn  <= {Pout_hs_dn[N-2:0],syn_off0_hs};
                Pout_vs_dn  <= {Pout_vs_dn[N-2:0],syn_off0_vs}; 
                Pout_de_dn  <= {Pout_de_dn[N-2:0],out_de}; 
            end
    end

//--------------------------------------------------
    assign O_adv7513_clk  = pixel_clk;
    assign O_adv7513_vs   = Pout_vs_dn[4];
    assign O_adv7513_hs   = Pout_hs_dn[4];
    assign O_adv7513_de   = Pout_de_dn[4];
    assign O_adv7513_data = off0_syn_de ? off0_syn_data : 24'h00007f;//{r,g,b}

//=========================================================================
// ADV7513 I2c 初始化
adv7513_iic_init adv7513_iic_init_inst
(
    .I_CLK              ( I_clk             ),
    .I_RESETN           ( rst_n1                ),
    .start              ( rst_n1                ),
    .O_TX_EN            ( TX_EN                 ),
    .O_WADDR            ( WADDR                 ),
    .O_WDATA            ( WDATA                 ),
    .O_RX_EN            ( RX_EN                 ),
    .O_RADDR            ( RADDR                 ),
    .I_RDATA            ( RDATA                 ),
    .cstate_flag        (                       ),
    .error_flag         (                       )
);

I2C_MASTER_Top I2C_MASTER_Top_inst
(
    .I_CLK              ( I_clk             ),
    .I_RESETN           ( rst_n1                ),
    .I_TX_EN            ( TX_EN                 ),
    .I_WADDR            ( WADDR                 ),
    .I_WDATA            ( WDATA                 ),
    .I_RX_EN            ( RX_EN                 ),
    .I_RADDR            ( RADDR                 ),
    .O_RDATA            ( RDATA                 ),
    .O_IIC_INT          (                       ),
    .SCL                ( IO_adv7513_scl        ),
    .SDA                ( IO_adv7513_sda        )
); 

endmodule